require File.expand_path(File.dirname(__FILE__) + '/../spec_config.rb')

describe Page do

  before do
    @page = Page.new(template: :default, title: 'Root')
    @child_page = Page.new(template: :default, title: 'Child')
    @grandchild_page = Page.new(template: :default, title: 'Grandchild')
    @default_fragments = ['Fragment 1', 'Fragment 2']
    @default_fragments.each do |label|
      @page.fragments << Fragment.new(label: label, body: 'test')
    end
    @attachment1 = File.open(Padrino.root('spec/fixtures/files/test1.jpg'))
    @attachment2 = File.open(Padrino.root('spec/fixtures/files/test2.jpg'))
  end

  after do
    FileUtils.rm_rf(Dir[Padrino.root('public/attachment', @page.id)])
    @page.assets = nil
    @page.destroy
    @child_page.destroy
    @grandchild_page.destroy
  end

  it 'should be a Mongo document' do
    @page.must be_document
  end

  it 'should have a template field' do
    @page.must have_field(:template).of_type(Symbol)
  end

  it 'should have slug field' do
    @page.must have_field(:slug)
  end

  it 'should have path field' do
    @page.must have_field(:path)
  end

  it 'should have title field' do
    @page.must have_field(:title)
  end

  it 'should have keywords field' do
    @page.must have_field(:keywords).of_type(Array)
  end

  it 'should have description field' do
    @page.must have_field(:description)
  end

  it 'embeds fragments' do
    @page.must embed_many(:fragments)
  end

  it 'returns fragments by label' do
    @page.find_fragment(@default_fragments.first).label.must_equal @default_fragments.first
  end

  it 'should add configured fragments' do
    @page.manage_fragments(['Fragment 1', 'Fragment 2', 'test fragment'])
    @page.find_fragment('test fragment').nil?.must_equal false
  end

  it 'should mark unconfigured fragments' do
    @page.manage_fragments(['Fragment 1'])
    @page.find_fragment('Fragment 2').active.must_equal false
  end

  it 'should only return active fragments' do
    @page.manage_fragments(['Fragment 1'])
    @page.active_fragments.size.must_equal 1
  end

  it 'should retrieve inactive fragments' do
    @page.manage_fragments(['Fragment 1'])
    @page.inactive_fragments.size.must_equal 1
  end

  it 'should just delete an inactive fragment if it\'s empty' do
    @page.fragments.first.body = ''
    @page.manage_fragments(['Fragment 2'])
    @page.inactive_fragments.size.must_equal 0
  end

  it 'should have page body' do
    @page.must have_field(:body)
  end

  it 'should reference child pagees' do
    @page.must have_many(:children)
  end

  it 'should allow child pagees' do
    @page.children << @child_page
    @page.valid?.must_equal true
  end

  it 'should allow grandchildren' do
    @child_page.children << @grandchild_page
    @page.children << @child_page
    @page.valid?.must_equal true
  end

  it 'should reference its parent when it is a child of another page' do
    @page.children << @child_page
    @page.children.first.parent.must_equal @page
  end

  it 'should have a poisition number' do
    @page.must have_field(:position).of_type(Integer).with_default_value(0)
  end

  it 'should have a chronological flag' do
    @page.must have_field(:chronological).of_type(Boolean).with_default_value(false)
  end

  it 'should have an index on path' do
    @page.must have_index_for(:path)
  end

  it 'should be valid with children when chronological' do
    @page.chronological = true
    @page.children << @child_page
    @page.valid?.must_equal true
  end

  it 'should not be valid with grandchildren when chronological' do
    @page.chronological = true
    @child_page.children << @grandchild_page
    @page.children << @child_page
    @page.valid?.must_equal false
  end

  it 'should not be valid with children if parent is chronological' do
    @page.chronological = true
    @child_page.children << @grandchild_page
    @page.children << @child_page
    @child_page.valid?.must_equal false
  end

  it 'should not identify itself as a post when parent is not chronological' do
    @page.chronological = false
    @page.children << @child_page
    @child_page.post?.must_equal false
  end

  it 'should identify itself as a post when parent is chronological' do
    @page.chronological = true
    @page.children << @child_page
    @child_page.post?.must_equal true
  end

  it 'should flag asset as used when an asset is referenced in body text' do
    @page.body = 'test1.jpg'
    @page.assets << Asset.new(caption: 'Test Asset', attachment: @attachment1)
    @page.save
    @page.assets.first.used?.must_equal true
  end

  it 'should flag asset as used when an asset is referenced in fragment body text' do
    @page.fragments << Fragment.new(label: 'Test Fragment', body: 'test1.jpg')
    @page.assets << Asset.new(caption: 'Test Asset', attachment: @attachment1)
    @page.save
    @page.assets.first.used?.must_equal true
  end

  it 'should not flag asset as used when an asset is not referenced in body or fragment body text' do
    @page.body = 'Test Text'
    @page.fragments << Fragment.new(label: 'Test Fragment', body: 'Test Text')
    @page.assets << Asset.new(caption: 'Test Asset', attachment: @attachment1)
    @page.save
    @page.assets.first.used?.must_equal false
  end

  it 'should return unused assets as a list' do
    asset1 = Asset.new(caption: 'Test Asset 1', attachment: @attachment1)
    asset2 = Asset.new(caption: 'Test Asset 2', attachment: @attachment2)
    @page.body = 'test2.jpg'
    @page.assets << asset1
    @page.assets << asset2
    @page.save
    @page.unused_assets.must_equal [asset1]
  end

  it 'should destroy assets marked for deletion' do
    asset1 = Asset.new(caption: 'Test Asset 1', attachment: @attachment1)
    asset2 = Asset.new(caption: 'Test Asset 2', attachment: @attachment2)
    @page.assets << asset1
    @page.assets << asset2
    @page.save
    @page.assets[1].delete_asset = true
    @page.save
    @page.assets.must_equal [asset1]
  end

  it 'should generate a slug based on page title' do
    @page.title = 'Root Page'
    @page.save
    @page.slug.must_equal 'root-page'
  end

  it 'should automatically generate a path for each page given its hierarchy' do
    @child_page.children << @grandchild_page
    @page.children << @child_page
    @page.save
    @grandchild_page.path.must_equal [@page.slug, @child_page.slug, @grandchild_page.slug].join('/')
  end

end
